from cstr_model import CSTRParameters, setup_cstr_model_equations
import numpy as np
from cstr_utils import plot_cstr
import time
from integrators import RK4Integrator, CVODESIntegrator


def simulate_open_loop(integrator, x0: np.ndarray, u_ref: np.ndarray, Nsim: int):

    nx = x0.shape[1]
    nu = u_ref.shape[1]

    X = np.ndarray((Nsim + 1, nx))
    U = np.ndarray((Nsim, nu))

    # open loop simulation
    x_current = x0
    X[[0]] = x_current

    start = time.time()

    for i in range(Nsim):

        x_next = integrator.simulate(x_current, u_ref).T

        x_current = x_next
        X[[i+1], :] = x_next
        U[[i], :] = u_ref

    simulation_time = time.time() - start

    return X, U, simulation_time


def main():

    Nsim = 80
    dt = 0.5  # [min]

    # initial state
    x0 = np.array([[1.141, 97, 0.8]])
    # x0 = np.array([[1.141, 97, 0.01]])

    u0 = np.array([[300, 0.101]])
    params = CSTRParameters()
    x, u, _, f = setup_cstr_model_equations(params)

    X_all = []
    U_all = []
    labels_all = []
    timings_all = []


    # Reference solution
    label = f"CVODES"
    print(f"\n\nRunning simulation with {label}.\n")

    integrator = CVODESIntegrator(x, u, f, dt)
    X_reference, U_reference, simulation_time = simulate_open_loop(integrator, x0, u0, Nsim)

    X_all.append(X_reference)
    U_all.append(U_reference)
    timings_all.append(simulation_time)
    labels_all.append(label)

    ax = None

    timings_RK4 = []
    error_RK4 = []
    h_RK4 = []

    for num_steps in [1, 10]:

        label = f"RK4 {num_steps=}"
        print(f"\n\nRunning simulation with {label}.\n")

        integrator = RK4Integrator(x, u, f, dt, num_steps=num_steps)
        X, U, simulation_time = simulate_open_loop(integrator, x0, u0, Nsim)

        diff_to_ref = np.max(np.abs(X - X_reference), axis=0)

        X_all.append(X)
        U_all.append(U)
        timings_all.append(simulation_time)
        labels_all.append(label)
        timings_RK4.append(simulation_time)
        error_RK4.append(np.max(diff_to_ref))
        h_RK4.append(dt/num_steps)


    # Evaluation
    print("\nTiming evaluation:\n------------------")
    for label, timing, X_traj in zip(labels_all, timings_all, X_all):

        diff_to_ref = np.max(np.abs(X_traj - X_reference), axis=0)
        print(f"{label}: {np.min(timing*1e3):.3f} ms, max diff: {diff_to_ref}\n")

    # plot results
    plot_cstr(dt, X_all, U_all, labels_all)


if __name__ == "__main__":
    main()
